@@ -2,7 +2,7 @@
  * misc.m: code not specific to vlc
  *****************************************************************************
  * Copyright (C) 2003-2011 VLC authors and VideoLAN
- * $Id: fc0b3bfc11915ad3cd48269cb5436d09ba61626a $
+ * $Id: bb8c687284dfb897c76e9b15bbd72b072a0167cc $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Felix Paul Kühne <fkuehne at videolan dot org>
@@ -92,7 +92,10 @@ static NSMutableArray *blackoutWindows = NULL;
 
 - (BOOL)isMainScreen
 {
-    return ([self displayID] == [[[NSScreen screens] objectAtIndex:0] displayID]);
+    if (OSX_REDACTED)
+        return YES;
+    else
+        return ([self displayID] == [[[NSScreen screens] objectAtIndex:0] displayID]);
 }
 
 - (BOOL)isScreen: (NSScreen*)screen
